System.register(['react'], (function (exports) {
  'use strict';
  var useState, useEffect;
  return {
    setters: [function (module) {
      useState = module.useState;
      useEffect = module.useEffect;
    }],
    execute: (function () {

      exports('createLocalStorageManager', createLocalStorageManager);

      const maybeWindow = typeof window === "undefined" ? void 0 : window;
      const localStorageChangeEventName = exports('localStorageChangeEventName', "localStorageChange");
      function createLocalStorageManager(key, validate, defaultValue) {
        const cache = {};
        const createLocalEvent = () => {
          return new CustomEvent(localStorageChangeEventName, {
            detail: cache
          });
        };
        const dispatchChange = () => {
          maybeWindow == null ? void 0 : maybeWindow.dispatchEvent(createLocalEvent());
        };
        const events = maybeWindow && new maybeWindow.EventTarget();
        const localListener = (event) => {
          const customEvent = event;
          if ("value" in customEvent.detail) {
            cache.value = customEvent.detail.value;
          } else {
            delete cache.value;
          }
          events == null ? void 0 : events.dispatchEvent(createLocalEvent());
        };
        const multiTabListener = () => {
          delete cache.value;
          events == null ? void 0 : events.dispatchEvent(createLocalEvent());
        };
        if (maybeWindow) {
          maybeWindow == null ? void 0 : maybeWindow.addEventListener(localStorageChangeEventName, localListener);
          maybeWindow == null ? void 0 : maybeWindow.addEventListener("storage", multiTabListener);
        }
        const store = {
          read() {
            if (maybeWindow === void 0)
              return defaultValue;
            const value = maybeWindow.localStorage.getItem(key);
            if (value === null) {
              return defaultValue;
            }
            try {
              const parsed = JSON.parse(value);
              return validate(parsed);
            } catch (_) {
              return defaultValue;
            }
          },
          get() {
            if (!("value" in cache)) {
              cache.value = store.read();
            }
            return cache.value;
          },
          set(data) {
            const newValue = data instanceof Function ? data(store.get()) : data;
            maybeWindow == null ? void 0 : maybeWindow.localStorage.setItem(key, JSON.stringify(newValue));
            cache.value = newValue;
            dispatchChange();
          },
          remove() {
            maybeWindow == null ? void 0 : maybeWindow.localStorage.removeItem(key);
            cache.value = defaultValue;
            dispatchChange();
          },
          watch(listener) {
            const wrappedListener = () => listener(store.get());
            events == null ? void 0 : events.addEventListener(localStorageChangeEventName, wrappedListener);
            return () => {
              events == null ? void 0 : events.removeEventListener(localStorageChangeEventName, wrappedListener);
            };
          },
          use() {
            const [value, setValue] = useState(store.get);
            useEffect(() => store.watch(setValue), []);
            return value;
          },
          destroy() {
            maybeWindow == null ? void 0 : maybeWindow.removeEventListener(localStorageChangeEventName, localListener);
            maybeWindow == null ? void 0 : maybeWindow.removeEventListener("storage", multiTabListener);
          }
        };
        return store;
      }

    })
  };
}));
