(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react')) :
  typeof define === 'function' && define.amd ? define(['exports', 'react'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.zustand = {}, global.react));
})(this, (function (exports, react) { 'use strict';

  var maybeWindow = typeof window === 'undefined' ? undefined : window;
  var localStorageChangeEventName = 'localStorageChange';
  function createLocalStorageManager(key, validate, defaultValue) {
    var cache = {};

    var createLocalEvent = function createLocalEvent() {
      return new CustomEvent(localStorageChangeEventName, {
        detail: cache
      });
    };

    var dispatchChange = function dispatchChange() {
      maybeWindow == null ? void 0 : maybeWindow.dispatchEvent(createLocalEvent());
    };

    var events = maybeWindow && new maybeWindow.EventTarget();

    var localListener = function localListener(event) {
      var customEvent = event;

      if ('value' in customEvent.detail) {
        cache.value = customEvent.detail.value;
      } else {
        delete cache.value;
      }

      events == null ? void 0 : events.dispatchEvent(createLocalEvent());
    };

    var multiTabListener = function multiTabListener() {
      delete cache.value;
      events == null ? void 0 : events.dispatchEvent(createLocalEvent());
    };

    if (maybeWindow) {
      maybeWindow == null ? void 0 : maybeWindow.addEventListener(localStorageChangeEventName, localListener);
      maybeWindow == null ? void 0 : maybeWindow.addEventListener('storage', multiTabListener);
    }

    var store = {
      read: function read() {
        if (maybeWindow === undefined) return defaultValue;
        var value = maybeWindow.localStorage.getItem(key);

        if (value === null) {
          return defaultValue;
        }

        try {
          var parsed = JSON.parse(value);
          return validate(parsed);
        } catch (_) {
          return defaultValue;
        }
      },
      get: function get() {
        if (!('value' in cache)) {
          cache.value = store.read();
        }

        return cache.value;
      },
      set: function set(data) {
        var newValue = data instanceof Function ? data(store.get()) : data;
        maybeWindow == null ? void 0 : maybeWindow.localStorage.setItem(key, JSON.stringify(newValue));
        cache.value = newValue;
        dispatchChange();
      },
      remove: function remove() {
        maybeWindow == null ? void 0 : maybeWindow.localStorage.removeItem(key);
        cache.value = defaultValue;
        dispatchChange();
      },
      watch: function watch(listener) {
        var wrappedListener = function wrappedListener() {
          return listener(store.get());
        };

        events == null ? void 0 : events.addEventListener(localStorageChangeEventName, wrappedListener);
        return function () {
          events == null ? void 0 : events.removeEventListener(localStorageChangeEventName, wrappedListener);
        };
      },
      use: function use() {
        var _useState = react.useState(store.get),
            value = _useState[0],
            setValue = _useState[1];

        react.useEffect(function () {
          return store.watch(setValue);
        }, []);
        return value;
      },
      destroy: function destroy() {
        maybeWindow == null ? void 0 : maybeWindow.removeEventListener(localStorageChangeEventName, localListener);
        maybeWindow == null ? void 0 : maybeWindow.removeEventListener('storage', multiTabListener);
      }
    };
    return store;
  }

  exports.createLocalStorageManager = createLocalStorageManager;
  exports.localStorageChangeEventName = localStorageChangeEventName;

  Object.defineProperty(exports, '__esModule', { value: true });

}));
